/**
 * @file catchError.ts
 * @description 通用的错误处理工具函数，支持异步和同步操作的错误捕获、重试、超时和日志记录。
 */
/**
 * @class TimeoutError
 * @description 表示操作超时的自定义错误类型。
 */
export declare class TimeoutError extends Error {
    constructor(message?: string);
}
/**
 * @type ErrorResult
 * @description 包装操作结果，可能包含成功时的数据或发生的错误，以及相关的元数据。
 * @template T - 成功时数据的类型。
 * @template E - 错误对象的类型，默认为 Error。
 */
export type ErrorResult<T, E extends Error = Error> = {
    data?: T;
    error?: E | TimeoutError;
    meta: {
        executionTime: number;
        retryCount?: number;
        retryErrors?: (E | TimeoutError)[];
        isTimeout?: boolean;
        [key: string]: unknown;
    };
};
/**
 * @type LogOnErrorFormatterParams
 * @description `logOnError` 自定义格式化函数的参数类型。
 * @template E - 错误对象的类型。
 */
export type LogOnErrorFormatterParams<E extends Error = Error> = {
    error: E | TimeoutError;
    meta: ErrorResult<unknown, E>["meta"];
    attempt?: number;
    maxAttempts?: number;
};
/**
 * @type LogOnErrorFormatterResult
 * @description `logOnError` 自定义格式化函数的返回值类型。
 * 可以是字符串、结构化日志对象，或者 null/undefined (用于禁止当前错误的日志记录)。
 */
export type LogOnErrorFormatterResult = string | {
    message: string;
    [key: string]: unknown;
} | null | undefined;
/**
 * @interface Logger
 * @description 定义了一个通用的日志记录器接口，以便用户可以集成自己的日志库。
 */
export interface Logger {
    error: (message: string, ...optionalParams: unknown[]) => void;
    warn: (message: string, ...optionalParams: unknown[]) => void;
    info: (message: string, ...optionalParams: unknown[]) => void;
    debug: (message: string, ...optionalParams: unknown[]) => void;
}
/**
 * @type CatchErrorOptions
 * @description `catchError` 和 `catchErrorSync` 函数的配置选项。
 * @template E - 错误对象的类型。
 */
export type CatchErrorOptions<E extends Error = Error> = {
    /**
     * 自定义日志记录器实例，默认为 `console`。
     */
    logger?: Logger;
    /**
     * 配置最终错误的自动日志记录。
     * - `true`: 使用默认消息记录日志。
     * - `string`: 使用该字符串作为默认日志消息的前缀。
     * - `(params: LogOnErrorFormatterParams<E>) => LogOnErrorFormatterResult`:
     * 一个函数，用于生成自定义日志消息或结构化日志对象。
     * 返回 `null` 或 `undefined` 以禁止记录此特定错误。
     * 默认为 `false` (不记录日志)。
     */
    logOnError?: boolean | string | ((params: LogOnErrorFormatterParams<E>) => LogOnErrorFormatterResult);
    /**
     * 自动记录错误时使用的日志级别。
     * 默认为 `'error'`。
     */
    logErrorSeverity?: "error" | "warn" | "info" | "debug";
    /**
     * 重试配置 (仅用于 `catchError` 异步函数)。
     */
    retry?: {
        /** 最大尝试次数 (包括第一次执行)。默认为 1 (不重试)。 */
        attempts: number;
        /**
         * 重试之间的延迟时间 (毫秒)。
         * 可以是一个固定数值，或者是一个函数，该函数接收当前尝试次数和错误对象，返回延迟时间。
         */
        delay?: number | ((attempt: number, error: E | TimeoutError) => number);
        /**
         * 一个函数，用于判断是否应该重试当前错误。
         * 返回 `true` 则重试，`false` 则不重试。
         * 如果未提供，则只要未达到最大尝试次数就会重试。
         */
        retryIf?: (error: E | TimeoutError) => boolean;
        /**
         * 每次重试尝试前调用的回调函数 (在延迟之后，下一次尝试之前)。
         */
        onProgress?: (progress: {
            attempt: number;
            maxAttempts: number;
            error: E | TimeoutError;
        }) => void;
    };
    /**
     * 操作的超时时间 (毫秒) (仅用于 `catchError` 异步函数)。
     * 如果操作在此时间内未完成，则会抛出 `TimeoutError`。
     */
    timeout?: number;
    /**
     * 无论操作成功或失败，最后都会执行的回调函数。
     */
    onFinally?: () => void;
};
/**
 * @function setDefaultCatchErrorOptions
 * @description 设置全局默认的 `catchError` 选项。
 * @param options - 要设置的默认选项。
 */
export declare function setDefaultCatchErrorOptions(options: Partial<CatchErrorOptions>): void;
/**
 * @function normalizeError
 * @description 将捕获到的各种类型的错误统一转换为标准的 Error 对象。
 * @param errorValue - 捕获到的原始错误，类型为 unknown。
 * @returns 标准的 Error 对象。
 */
export declare function normalizeError(errorValue: unknown): Error;
/**
 * @function catchError
 * @description 捕获 Promise 执行过程中的错误，并提供重试、超时和日志等功能。
 * @template TData - Promise 函数成功时解析的数据类型。
 * @template TResult - 返回结果中 `data` 字段的类型，默认为 `TData`。
 * @template E - 期望捕获的错误类型，默认为 `Error`。
 * @param promiseFn - 一个返回 Promise 的函数，代表要执行的异步操作。
 * @param options - `CatchErrorOptions` 配置选项。
 * @returns `Promise<ErrorResult<TResult, E>>` - 包含数据或错误以及元信息的结果。
 * @example
 * // 基本用法：成功
 * const { data, error, meta } = await catchError(async () => someAsyncOperation());
 * if (data) console.log('Success:', data);
 *
 * @example
 * // 基本用法：失败
 * const { data: data2, error: error2 } = await catchError(async () => {
 * throw new Error("Something went wrong");
 * });
 * if (error2) console.error('Failed:', error2.message, error2.name);
 *
 * @example
 * // 使用重试 (尝试3次，每次延迟100ms)
 * const { error: error3 } = await catchError(async () => flakyAsyncOperation(), {
 * retry: {
 * attempts: 3,
 * delay: 100,
 * onProgress: ({ attempt, maxAttempts }) => console.log(`Retrying ${attempt}/${maxAttempts}...`)
 * },
 * logOnError: true // 最终失败时自动记录日志
 * });
 * if (error3) console.error('Still failed after retries:', error3.message);
 *
 * @example
 * // 使用超时 (500ms)
 * const { error: error4, meta: meta4 } = await catchError(async () => longRunningOperation(), {
 * timeout: 500,
 * logOnError: "Operation timed out:"
 * });
 * if (error4 && meta4.isTimeout) console.error('Timeout occurred:', error4.message);
 *
 * @example
 * // 自定义日志格式
 * const { error: error5 } = await catchError(async () => anotherAsyncOp(), {
 * logOnError: (params) => ({
 * message: `Custom log for ${params.error.name}`,
 * errorCode: (params.error as any).code || 'UNKNOWN', // 假设错误对象可能有 code 属性
 * attempt: params.attempt,
 * meta: params.meta
 * }),
 * logErrorSeverity: 'warn'
 * });
 *
 * @example
 * // 不记录特定错误
 * const { error: error6 } = await catchError(async () => sensitiveOperation(), {
 * logOnError: (params) => {
 * if (params.error.message === 'ignore_this') return null; // 不记录此错误
 * return "An error occurred in sensitive operation";
 * }
 * });
 */
export declare function catchError<TData, TResult = TData, E extends Error = Error>(promiseFn: () => Promise<TData>, options?: CatchErrorOptions<E>): Promise<ErrorResult<TResult, E>>;
/**
 * @function catchErrorSync
 * @description 同步版本的 `catchError`，用于捕获同步函数执行中的错误。
 * @template T - 函数成功时的返回值类型。
 * @template E - 期望捕获的错误类型，默认为 `Error`。
 * @param fn - 要执行的同步函数。
 * @param options - `CatchErrorOptions` 配置选项 (不包括异步相关的 "timeout", "retry", "onProgress")。
 * @returns `ErrorResult<T, E>` - 包含数据或错误以及元信息的结果。
 * @example
 * // 基本用法：成功
 * const { data, error } = catchErrorSync(() => someSyncOperation());
 * if (data) console.log('Success:', data);
 *
 * @example
 * // 基本用法：失败
 * const { error: error2 } = catchErrorSync(() => {
 * throw new Error("Sync failed");
 * });
 * if (error2) console.error('Failed:', error2.message);
 *
 * @example
 * // 自动记录错误日志
 * const { error: error3 } = catchErrorSync(() => riskySyncOperation(), {
 * logOnError: true
 * });
 *
 * @example
 * // 自定义日志消息前缀和级别
 * const { error: error4 } = catchErrorSync(() => anotherSync(), {
 * logOnError: "Critical sync error:",
 * logErrorSeverity: 'warn'
 * });
 *
 * @example
 * // 使用 onFinally 回调
 * let cleanedUp = false;
 * const { data: data5 } = catchErrorSync(() => {
 * console.log("Performing sync task...");
 * return "sync data";
 * }, {
 * onFinally: () => {
 * cleanedUp = true;
 * console.log("Sync cleanup done.");
 * }
 * });
 * console.log("Cleaned up:", cleanedUp);
 */
export declare function catchErrorSync<T, E extends Error = Error>(fn: () => T, options?: Omit<CatchErrorOptions<E>, "timeout" | "retry" | "onProgress">): ErrorResult<T, E>;
