"use strict";
/**
 * @file catchError.ts
 * @description 通用的错误处理工具函数，支持异步和同步操作的错误捕获、重试、超时和日志记录。
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.TimeoutError = void 0;
exports.setDefaultCatchErrorOptions = setDefaultCatchErrorOptions;
exports.normalizeError = normalizeError;
exports.catchError = catchError;
exports.catchErrorSync = catchErrorSync;
// --- 核心类型定义 ---
/**
 * @class TimeoutError
 * @description 表示操作超时的自定义错误类型。
 */
class TimeoutError extends Error {
    constructor(message = "Operation timed out") {
        super(message);
        this.name = "TimeoutError";
        // 在 TypeScript 中，当目标是 ES5 或更低版本时，需要手动设置原型链
        Object.setPrototypeOf(this, TimeoutError.prototype);
    }
}
exports.TimeoutError = TimeoutError;
// --- 全局配置 ---
let globalCatchErrorOptions = {
    // E 默认为 Error
    logger: console,
    logOnError: false,
    logErrorSeverity: "error",
};
/**
 * @function setDefaultCatchErrorOptions
 * @description 设置全局默认的 `catchError` 选项。
 * @param options - 要设置的默认选项。
 */
function setDefaultCatchErrorOptions(options) {
    // E 默认为 Error
    globalCatchErrorOptions = { ...globalCatchErrorOptions, ...options };
}
// --- 辅助函数 ---
/**
 * @function normalizeError
 * @description 将捕获到的各种类型的错误统一转换为标准的 Error 对象。
 * @param errorValue - 捕获到的原始错误，类型为 unknown。
 * @returns 标准的 Error 对象。
 */
function normalizeError(errorValue) {
    if (errorValue instanceof Error) {
        return errorValue;
    }
    if (typeof errorValue === "string") {
        return new Error(errorValue);
    }
    if (typeof errorValue === "object" &&
        errorValue !== null &&
        "message" in errorValue) {
        const message = String(errorValue.message ||
            "An unknown error occurred");
        const newError = new Error(message);
        if ("name" in errorValue &&
            typeof errorValue.name === "string") {
            newError.name = errorValue.name;
        }
        return newError;
    }
    return new Error("An unknown error occurred");
}
// --- 核心错误处理函数 ---
/**
 * @function catchError
 * @description 捕获 Promise 执行过程中的错误，并提供重试、超时和日志等功能。
 * @template TData - Promise 函数成功时解析的数据类型。
 * @template TResult - 返回结果中 `data` 字段的类型，默认为 `TData`。
 * @template E - 期望捕获的错误类型，默认为 `Error`。
 * @param promiseFn - 一个返回 Promise 的函数，代表要执行的异步操作。
 * @param options - `CatchErrorOptions` 配置选项。
 * @returns `Promise<ErrorResult<TResult, E>>` - 包含数据或错误以及元信息的结果。
 * @example
 * // 基本用法：成功
 * const { data, error, meta } = await catchError(async () => someAsyncOperation());
 * if (data) console.log('Success:', data);
 *
 * @example
 * // 基本用法：失败
 * const { data: data2, error: error2 } = await catchError(async () => {
 * throw new Error("Something went wrong");
 * });
 * if (error2) console.error('Failed:', error2.message, error2.name);
 *
 * @example
 * // 使用重试 (尝试3次，每次延迟100ms)
 * const { error: error3 } = await catchError(async () => flakyAsyncOperation(), {
 * retry: {
 * attempts: 3,
 * delay: 100,
 * onProgress: ({ attempt, maxAttempts }) => console.log(`Retrying ${attempt}/${maxAttempts}...`)
 * },
 * logOnError: true // 最终失败时自动记录日志
 * });
 * if (error3) console.error('Still failed after retries:', error3.message);
 *
 * @example
 * // 使用超时 (500ms)
 * const { error: error4, meta: meta4 } = await catchError(async () => longRunningOperation(), {
 * timeout: 500,
 * logOnError: "Operation timed out:"
 * });
 * if (error4 && meta4.isTimeout) console.error('Timeout occurred:', error4.message);
 *
 * @example
 * // 自定义日志格式
 * const { error: error5 } = await catchError(async () => anotherAsyncOp(), {
 * logOnError: (params) => ({
 * message: `Custom log for ${params.error.name}`,
 * errorCode: (params.error as any).code || 'UNKNOWN', // 假设错误对象可能有 code 属性
 * attempt: params.attempt,
 * meta: params.meta
 * }),
 * logErrorSeverity: 'warn'
 * });
 *
 * @example
 * // 不记录特定错误
 * const { error: error6 } = await catchError(async () => sensitiveOperation(), {
 * logOnError: (params) => {
 * if (params.error.message === 'ignore_this') return null; // 不记录此错误
 * return "An error occurred in sensitive operation";
 * }
 * });
 */
async function catchError(promiseFn, options) {
    const mergedOptions = {
        ...globalCatchErrorOptions,
        ...options,
    }; // 类型断言仍然需要，因为 E 可能与 global 不同
    const { logger, logOnError, logErrorSeverity, retry, timeout, onFinally } = mergedOptions;
    const currentLogger = logger || console; // Logger 类型是明确的
    const severity = logErrorSeverity || "error";
    const meta = {
        executionTime: 0,
        retryErrors: [],
        isTimeout: false,
    };
    const startTime = Date.now();
    let attempt = 0;
    const maxAttempts = (retry === null || retry === void 0 ? void 0 : retry.attempts) || 1;
    const handleFinalErrorLogging = (err, currentMeta) => {
        if (!logOnError || !currentLogger[severity]) {
            // currentLogger[severity] 是类型安全的
            return;
        }
        let logOutput;
        if (typeof logOnError === "function") {
            logOutput = logOnError({
                error: err,
                meta: currentMeta,
                attempt,
                maxAttempts,
            });
        }
        else {
            const prefix = typeof logOnError === "string" ? logOnError : "Async operation failed:";
            logOutput = {
                message: `${prefix} ${err.message}`,
                error: err,
                meta: currentMeta,
            };
        }
        if (logOutput === null || logOutput === undefined)
            return;
        if (typeof logOutput === "string") {
            currentLogger[severity](logOutput, { error: err, meta: currentMeta });
        }
        else if (typeof logOutput === "object" && logOutput.message) {
            const { message, ...extraData } = logOutput;
            const logArgs = { error: err, meta: currentMeta, ...extraData };
            currentLogger[severity](message, logArgs);
        }
    };
    try {
        // eslint-disable-next-line no-constant-condition
        while (true) {
            attempt++;
            let attemptPromise = promiseFn();
            if (timeout && timeout > 0) {
                attemptPromise = Promise.race([
                    attemptPromise,
                    new Promise((_, reject) => setTimeout(() => {
                        meta.isTimeout = true;
                        reject(new TimeoutError(`Operation timed out after ${timeout}ms`));
                    }, timeout)),
                ]);
            }
            try {
                const result = await attemptPromise;
                meta.executionTime = Date.now() - startTime;
                if (attempt > 1)
                    meta.retryCount = attempt - 1;
                return { data: result, meta };
            }
            catch (errorValue) {
                // errorValue is unknown here
                const err = normalizeError(errorValue); // Cast after normalization
                if (meta.retryErrors)
                    meta.retryErrors.push(err);
                if (meta.isTimeout) {
                    meta.executionTime = Date.now() - startTime;
                    if (attempt > 1 && retry)
                        meta.retryCount = attempt - 1;
                    handleFinalErrorLogging(err, meta);
                    return { error: err, meta };
                }
                if (retry &&
                    attempt < maxAttempts &&
                    (!retry.retryIf || retry.retryIf(err))) {
                    if (retry.onProgress) {
                        try {
                            retry.onProgress({ attempt, maxAttempts, error: err });
                        }
                        catch (onProgressErrorValue) {
                            console.error("Error in onProgress callback:", normalizeError(onProgressErrorValue));
                        }
                    }
                    if (retry.delay) {
                        const delayDuration = typeof retry.delay === "function"
                            ? retry.delay(attempt, err)
                            : retry.delay;
                        if (delayDuration > 0) {
                            await new Promise((resolve) => setTimeout(resolve, delayDuration));
                        }
                    }
                    // 继续下一次尝试
                }
                else {
                    meta.executionTime = Date.now() - startTime;
                    if (attempt > 1 && retry)
                        meta.retryCount = attempt - 1;
                    handleFinalErrorLogging(err, meta);
                    return { error: err, meta };
                }
            }
        }
    }
    catch (unexpectedErrorValue) {
        // unexpectedErrorValue is unknown
        const err = normalizeError(unexpectedErrorValue);
        meta.executionTime = Date.now() - startTime;
        handleFinalErrorLogging(err, meta);
        return { error: err, meta };
    }
    finally {
        if (onFinally) {
            try {
                onFinally();
            }
            catch (e) {
                console.error("Error in onFinally callback:", normalizeError(e));
            }
        }
    }
}
/**
 * @function catchErrorSync
 * @description 同步版本的 `catchError`，用于捕获同步函数执行中的错误。
 * @template T - 函数成功时的返回值类型。
 * @template E - 期望捕获的错误类型，默认为 `Error`。
 * @param fn - 要执行的同步函数。
 * @param options - `CatchErrorOptions` 配置选项 (不包括异步相关的 "timeout", "retry", "onProgress")。
 * @returns `ErrorResult<T, E>` - 包含数据或错误以及元信息的结果。
 * @example
 * // 基本用法：成功
 * const { data, error } = catchErrorSync(() => someSyncOperation());
 * if (data) console.log('Success:', data);
 *
 * @example
 * // 基本用法：失败
 * const { error: error2 } = catchErrorSync(() => {
 * throw new Error("Sync failed");
 * });
 * if (error2) console.error('Failed:', error2.message);
 *
 * @example
 * // 自动记录错误日志
 * const { error: error3 } = catchErrorSync(() => riskySyncOperation(), {
 * logOnError: true
 * });
 *
 * @example
 * // 自定义日志消息前缀和级别
 * const { error: error4 } = catchErrorSync(() => anotherSync(), {
 * logOnError: "Critical sync error:",
 * logErrorSeverity: 'warn'
 * });
 *
 * @example
 * // 使用 onFinally 回调
 * let cleanedUp = false;
 * const { data: data5 } = catchErrorSync(() => {
 * console.log("Performing sync task...");
 * return "sync data";
 * }, {
 * onFinally: () => {
 * cleanedUp = true;
 * console.log("Sync cleanup done.");
 * }
 * });
 * console.log("Cleaned up:", cleanedUp);
 */
function catchErrorSync(fn, options) {
    const mergedOptions = { ...globalCatchErrorOptions, ...options };
    const { logger, logOnError, logErrorSeverity, onFinally } = mergedOptions;
    const currentLogger = logger || console;
    const severity = logErrorSeverity || "error";
    const meta = { executionTime: 0 };
    const startTime = Date.now();
    const handleSyncErrorLogging = (err, currentMeta) => {
        if (!logOnError || !currentLogger[severity]) {
            return;
        }
        const formatterParams = {
            error: err,
            meta: currentMeta,
        };
        let logOutput;
        if (typeof logOnError === "function") {
            logOutput = logOnError(formatterParams);
        }
        else {
            const prefix = typeof logOnError === "string" ? logOnError : "Sync operation failed:";
            logOutput = {
                message: `${prefix} ${err.message}`,
                error: err,
                meta: currentMeta,
            };
        }
        if (logOutput === null || logOutput === undefined)
            return;
        if (typeof logOutput === "string") {
            currentLogger[severity](logOutput, { error: err, meta: currentMeta });
        }
        else if (typeof logOutput === "object" && logOutput.message) {
            const { message, ...extraData } = logOutput;
            const logArgs = { error: err, meta: currentMeta, ...extraData };
            currentLogger[severity](message, logArgs);
        }
    };
    try {
        const result = fn();
        meta.executionTime = Date.now() - startTime;
        return { data: result, meta };
    }
    catch (errorValue) {
        // errorValue is unknown here
        const err = normalizeError(errorValue); // Cast after normalization
        meta.executionTime = Date.now() - startTime;
        handleSyncErrorLogging(err, meta);
        return { error: err, meta };
    }
    finally {
        if (onFinally) {
            try {
                onFinally();
            }
            catch (e) {
                console.error("Error in onFinally (sync) callback:", normalizeError(e));
            }
        }
    }
}
